#![allow(non_snake_case)]

use vulkan_sys::prelude::*;

use crate::dummy_functions::*;

#[derive(Clone, Default)]
pub struct VkLayerInstanceDispatchTable {
    // Manually add in GetPhysicalDeviceProcAddr entry
    pub GetPhysicalDeviceProcAddr: Option<PFN_GetPhysicalDeviceProcAddr>,

    // ---- Core 1_0 commands
    pub CreateInstance: Option<PFN_vkCreateInstance>,
    pub DestroyInstance: Option<PFN_vkDestroyInstance>,
    pub EnumeratePhysicalDevices: Option<PFN_vkEnumeratePhysicalDevices>,
    pub GetPhysicalDeviceFeatures: Option<PFN_vkGetPhysicalDeviceFeatures>,
    pub GetPhysicalDeviceFormatProperties: Option<PFN_vkGetPhysicalDeviceFormatProperties>,
    pub GetPhysicalDeviceImageFormatProperties:
        Option<PFN_vkGetPhysicalDeviceImageFormatProperties>,
    pub GetPhysicalDeviceProperties: Option<PFN_vkGetPhysicalDeviceProperties>,
    pub GetPhysicalDeviceQueueFamilyProperties:
        Option<PFN_vkGetPhysicalDeviceQueueFamilyProperties>,
    pub GetPhysicalDeviceMemoryProperties: Option<PFN_vkGetPhysicalDeviceMemoryProperties>,
    pub GetInstanceProcAddr: Option<PFN_vkGetInstanceProcAddr>,
    pub CreateDevice: Option<PFN_vkCreateDevice>,
    pub EnumerateInstanceExtensionProperties: Option<PFN_vkEnumerateInstanceExtensionProperties>,
    pub EnumerateDeviceExtensionProperties: Option<PFN_vkEnumerateDeviceExtensionProperties>,
    pub EnumerateInstanceLayerProperties: Option<PFN_vkEnumerateInstanceLayerProperties>,
    pub EnumerateDeviceLayerProperties: Option<PFN_vkEnumerateDeviceLayerProperties>,
    pub GetPhysicalDeviceSparseImageFormatProperties:
        Option<PFN_vkGetPhysicalDeviceSparseImageFormatProperties>,

    // ---- Core 1_1 commands
    pub EnumerateInstanceVersion: Option<PFN_vkEnumerateInstanceVersion>,
    pub EnumeratePhysicalDeviceGroups: Option<PFN_vkEnumeratePhysicalDeviceGroups>,
    pub GetPhysicalDeviceFeatures2: Option<PFN_vkGetPhysicalDeviceFeatures2>,
    pub GetPhysicalDeviceProperties2: Option<PFN_vkGetPhysicalDeviceProperties2>,
    pub GetPhysicalDeviceFormatProperties2: Option<PFN_vkGetPhysicalDeviceFormatProperties2>,
    pub GetPhysicalDeviceImageFormatProperties2:
        Option<PFN_vkGetPhysicalDeviceImageFormatProperties2>,
    pub GetPhysicalDeviceQueueFamilyProperties2:
        Option<PFN_vkGetPhysicalDeviceQueueFamilyProperties2>,
    pub GetPhysicalDeviceMemoryProperties2: Option<PFN_vkGetPhysicalDeviceMemoryProperties2>,
    pub GetPhysicalDeviceSparseImageFormatProperties2:
        Option<PFN_vkGetPhysicalDeviceSparseImageFormatProperties2>,
    pub GetPhysicalDeviceExternalBufferProperties:
        Option<PFN_vkGetPhysicalDeviceExternalBufferProperties>,
    pub GetPhysicalDeviceExternalFenceProperties:
        Option<PFN_vkGetPhysicalDeviceExternalFenceProperties>,
    pub GetPhysicalDeviceExternalSemaphoreProperties:
        Option<PFN_vkGetPhysicalDeviceExternalSemaphoreProperties>,

    // ---- Core 1_3 commands
    pub GetPhysicalDeviceToolProperties: Option<PFN_vkGetPhysicalDeviceToolProperties>,

    // ---- VK_KHR_surface extension commands
    pub DestroySurfaceKHR: Option<PFN_vkDestroySurfaceKHR>,
    pub GetPhysicalDeviceSurfaceSupportKHR: Option<PFN_vkGetPhysicalDeviceSurfaceSupportKHR>,
    pub GetPhysicalDeviceSurfaceCapabilitiesKHR:
        Option<PFN_vkGetPhysicalDeviceSurfaceCapabilitiesKHR>,
    pub GetPhysicalDeviceSurfaceFormatsKHR: Option<PFN_vkGetPhysicalDeviceSurfaceFormatsKHR>,
    pub GetPhysicalDeviceSurfacePresentModesKHR:
        Option<PFN_vkGetPhysicalDeviceSurfacePresentModesKHR>,

    // ---- VK_KHR_swapchain extension commands
    pub GetPhysicalDevicePresentRectanglesKHR: Option<PFN_vkGetPhysicalDevicePresentRectanglesKHR>,

    // ---- VK_KHR_display extension commands
    pub GetPhysicalDeviceDisplayPropertiesKHR: Option<PFN_vkGetPhysicalDeviceDisplayPropertiesKHR>,
    pub GetPhysicalDeviceDisplayPlanePropertiesKHR:
        Option<PFN_vkGetPhysicalDeviceDisplayPlanePropertiesKHR>,
    pub GetDisplayPlaneSupportedDisplaysKHR: Option<PFN_vkGetDisplayPlaneSupportedDisplaysKHR>,
    pub GetDisplayModePropertiesKHR: Option<PFN_vkGetDisplayModePropertiesKHR>,
    pub CreateDisplayModeKHR: Option<PFN_vkCreateDisplayModeKHR>,
    pub GetDisplayPlaneCapabilitiesKHR: Option<PFN_vkGetDisplayPlaneCapabilitiesKHR>,
    pub CreateDisplayPlaneSurfaceKHR: Option<PFN_vkCreateDisplayPlaneSurfaceKHR>,

    // ---- VK_KHR_get_physical_device_properties2 extension commands
    pub GetPhysicalDeviceFeatures2KHR: Option<PFN_vkGetPhysicalDeviceFeatures2KHR>,
    pub GetPhysicalDeviceProperties2KHR: Option<PFN_vkGetPhysicalDeviceProperties2KHR>,
    pub GetPhysicalDeviceFormatProperties2KHR: Option<PFN_vkGetPhysicalDeviceFormatProperties2KHR>,
    pub GetPhysicalDeviceImageFormatProperties2KHR:
        Option<PFN_vkGetPhysicalDeviceImageFormatProperties2KHR>,
    pub GetPhysicalDeviceQueueFamilyProperties2KHR:
        Option<PFN_vkGetPhysicalDeviceQueueFamilyProperties2KHR>,
    pub GetPhysicalDeviceMemoryProperties2KHR: Option<PFN_vkGetPhysicalDeviceMemoryProperties2KHR>,
    pub GetPhysicalDeviceSparseImageFormatProperties2KHR:
        Option<PFN_vkGetPhysicalDeviceSparseImageFormatProperties2KHR>,

    // ---- VK_KHR_device_group_creation extension commands
    pub EnumeratePhysicalDeviceGroupsKHR: Option<PFN_vkEnumeratePhysicalDeviceGroupsKHR>,

    // ---- VK_KHR_external_memory_capabilities extension commands
    pub GetPhysicalDeviceExternalBufferPropertiesKHR:
        Option<PFN_vkGetPhysicalDeviceExternalBufferPropertiesKHR>,

    // ---- VK_KHR_external_semaphore_capabilities extension commands
    pub GetPhysicalDeviceExternalSemaphorePropertiesKHR:
        Option<PFN_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR>,

    // ---- VK_KHR_external_fence_capabilities extension commands
    pub GetPhysicalDeviceExternalFencePropertiesKHR:
        Option<PFN_vkGetPhysicalDeviceExternalFencePropertiesKHR>,

    // ---- VK_KHR_performance_query extension commands
    pub EnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR:
        Option<PFN_vkEnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR>,
    pub GetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR:
        Option<PFN_vkGetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR>,

    // ---- VK_KHR_get_surface_capabilities2 extension commands
    pub GetPhysicalDeviceSurfaceCapabilities2KHR:
        Option<PFN_vkGetPhysicalDeviceSurfaceCapabilities2KHR>,
    pub GetPhysicalDeviceSurfaceFormats2KHR: Option<PFN_vkGetPhysicalDeviceSurfaceFormats2KHR>,

    // ---- VK_KHR_get_display_properties2 extension commands
    pub GetPhysicalDeviceDisplayProperties2KHR:
        Option<PFN_vkGetPhysicalDeviceDisplayProperties2KHR>,
    pub GetPhysicalDeviceDisplayPlaneProperties2KHR:
        Option<PFN_vkGetPhysicalDeviceDisplayPlaneProperties2KHR>,
    pub GetDisplayModeProperties2KHR: Option<PFN_vkGetDisplayModeProperties2KHR>,
    pub GetDisplayPlaneCapabilities2KHR: Option<PFN_vkGetDisplayPlaneCapabilities2KHR>,

    // ---- VK_KHR_fragment_shading_rate extension commands
    pub GetPhysicalDeviceFragmentShadingRatesKHR:
        Option<PFN_vkGetPhysicalDeviceFragmentShadingRatesKHR>,

    // ---- VK_EXT_debug_report extension commands
    pub CreateDebugReportCallbackEXT: Option<PFN_vkCreateDebugReportCallbackEXT>,
    pub DestroyDebugReportCallbackEXT: Option<PFN_vkDestroyDebugReportCallbackEXT>,
    pub DebugReportMessageEXT: Option<PFN_vkDebugReportMessageEXT>,

    // ---- VK_NV_external_memory_capabilities extension commands
    pub GetPhysicalDeviceExternalImageFormatPropertiesNV:
        Option<PFN_vkGetPhysicalDeviceExternalImageFormatPropertiesNV>,

    // ---- VK_EXT_display_surface_counter extension commands
    pub GetPhysicalDeviceSurfaceCapabilities2EXT:
        Option<PFN_vkGetPhysicalDeviceSurfaceCapabilities2EXT>,

    // ---- VK_EXT_debug_utils extension commands
    pub CreateDebugUtilsMessengerEXT: Option<PFN_vkCreateDebugUtilsMessengerEXT>,
    pub DestroyDebugUtilsMessengerEXT: Option<PFN_vkDestroyDebugUtilsMessengerEXT>,
    pub SubmitDebugUtilsMessageEXT: Option<PFN_vkSubmitDebugUtilsMessageEXT>,

    // ---- VK_EXT_sample_locations extension commands
    pub GetPhysicalDeviceMultisamplePropertiesEXT:
        Option<PFN_vkGetPhysicalDeviceMultisamplePropertiesEXT>,

    // ---- VK_EXT_calibrated_timestamps extension commands
    pub GetPhysicalDeviceCalibrateableTimeDomainsEXT:
        Option<PFN_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT>,

    // ---- VK_EXT_tooling_info extension commands
    pub GetPhysicalDeviceToolPropertiesEXT: Option<PFN_vkGetPhysicalDeviceToolPropertiesEXT>,

    // ---- VK_NV_cooperative_matrix extension commands
    pub GetPhysicalDeviceCooperativeMatrixPropertiesNV:
        Option<PFN_vkGetPhysicalDeviceCooperativeMatrixPropertiesNV>,

    // ---- VK_NV_coverage_reduction_mode extension commands
    pub GetPhysicalDeviceSupportedFramebufferMixedSamplesCombinationsNV:
        Option<PFN_vkGetPhysicalDeviceSupportedFramebufferMixedSamplesCombinationsNV>,

    // ---- VK_EXT_headless_surface extension commands
    pub CreateHeadlessSurfaceEXT: Option<PFN_vkCreateHeadlessSurfaceEXT>,

    // ---- VK_EXT_acquire_drm_display extension commands
    pub AcquireDrmDisplayEXT: Option<PFN_vkAcquireDrmDisplayEXT>,
    pub GetDrmDisplayEXT: Option<PFN_vkGetDrmDisplayEXT>,

    // ---- VK_NV_optical_flow extension commands
    pub GetPhysicalDeviceOpticalFlowImageFormatsNV:
        Option<PFN_vkGetPhysicalDeviceOpticalFlowImageFormatsNV>,
}
